@@ -1,9 +1,9 @@
-module Ag (uuagcLib, uuagcExe) where
+-- Todo: we should make a nicer pipeline. Perhaps use Atze's "compile run" combinators.
+module Ag (uuagcLib, uuagcExe,compile) where
 
 import System.Environment            (getArgs, getProgName)
-import System.Exit                   (exitFailure)
 import System.Console.GetOpt         (usageInfo)
-import Data.List                     (isSuffixOf,nub,partition)
+import Data.List                     (partition)
 import Control.Monad                 (zipWithM_)
 import Data.Maybe
 import System.FilePath
@@ -13,6 +13,7 @@ import qualified Data.Map as Map
 import qualified Data.Sequence as Seq ((><),null)
 import Data.Foldable(toList)
 import Pretty
+import PPUtil
 
 import UU.Parsing                    (Message(..), Action(..))
 import UU.Scanner.Position           (Pos, line, file)
@@ -27,6 +28,7 @@ import qualified KWOrder            as Pass3a (sem_Grammar,  wrap_Grammar,  Syn_
 import qualified GenerateCode       as Pass4  (sem_CGrammar, wrap_CGrammar, Syn_CGrammar(..), Inh_CGrammar(..))
 import qualified PrintVisitCode     as Pass4a (sem_CGrammar, wrap_CGrammar, Syn_CGrammar(..), Inh_CGrammar(..))
 import qualified ExecutionPlan2Hs   as Pass4b (sem_ExecutionPlan, wrap_ExecutionPlan, Syn_ExecutionPlan(..), Inh_ExecutionPlan(..), warrenFlagsPP)
+import qualified ExecutionPlan2Caml as Pass4c (sem_ExecutionPlan, wrap_ExecutionPlan, Syn_ExecutionPlan(..), Inh_ExecutionPlan(..))
 import qualified PrintCode          as Pass5  (sem_Program,  wrap_Program,  Syn_Program (..), Inh_Program (..))
 import qualified PrintOcamlCode     as Pass5a (sem_Program,  wrap_Program,  Syn_Program (..), Inh_Program (..))
 import qualified PrintErrorMessages as PrErr  (sem_Errors ,  wrap_Errors ,  Syn_Errors  (..), Inh_Errors  (..), isError)
@@ -40,15 +42,15 @@ import qualified AG2AspectAG as AspectAGDump (pragmaAspectAG, sem_Grammar,  wrap
 import Options
 import Version       (banner)
 import Parser        (parseAG, depsAG, parseAGI)
-import ErrorMessages (Error(ParserError), Errors)
+import ErrorMessages (Error(ParserError))
 import CommonTypes
 import ATermWrite
 
 -- Library version
-import System.Exit (ExitCode(..))
+import System.Exit (ExitCode(..), exitWith)
 
 uuagcLib :: [String] -> FilePath -> IO (ExitCode, [FilePath])
-uuagcLib args file
+uuagcLib args fileP
   = do let (flags,_,errs) = getOptions args
        if showVersion flags || showHelp flags
          then do putStrLn "Cannot display help or version in library mode."
@@ -58,9 +60,9 @@ uuagcLib args file
                       mapM_ putStrLn errs
                       return (ExitFailure 2, [])
               else if genFileDeps flags
-                   then do deps <- getDeps flags [file]
+                   then do deps <- getDeps flags [fileP]
                            return (ExitSuccess, deps)
-                   else do compile flags file (head $ outputFiles flags++repeat "")
+                   else do compile flags fileP (head $ outputFiles flags++repeat "")
                            return (ExitSuccess, [])
 
 -- Executable version
@@ -73,19 +75,24 @@ uuagcExe
           (flags,files,errs) = getOptions args
 
       if showVersion flags
-       then putStrLn banner
-       else if null files || showHelp flags || (not.null) errs
-       then mapM_ putStrLn (usageInfo usageheader options : errs)
-       else if genFileDeps flags
-            then reportDeps flags files
-            else zipWithM_ (compile flags) files (outputFiles flags++repeat "")
+        then putStrLn banner
+        else if showHelp flags
+             then putStrLn (usageInfo usageheader options)
+             else if null files || (not.null) errs
+                  then do mapM_ putStrLn (usageInfo usageheader options : errs)
+                          exitWith (ExitFailure 2)
+                  else if genFileDeps flags
+                       then reportDeps flags files
+                       else zipWithM_ (compile flags) files (outputFiles flags++repeat "")
 
 
-compile :: Options -> String -> String -> IO ()
+compile :: Options -> FilePath -> FilePath -> IO ()
 compile flags input output
- = do (output0,parseErrors) <- parseAG flags (searchPath flags) (inputFile input)
+ = do (output0,parseErrors) <- parseAG flags (searchPath flags) input
       irrefutableMap <- readIrrefutableMap flags
-
+      let printStr  = outputStr flags
+          failWith  = failWithCode flags
+          inputfile = maybe input id (mainFilename flags)
       let output1   = Pass1.wrap_AG              (Pass1.sem_AG                                 output0 ) Pass1.Inh_AG       {Pass1.options_Inh_AG       = flags}
           flags'    = condDisableOptimizations (Pass1.pragmas_Syn_AG output1 flags)
           grammar1  = Pass1.output_Syn_AG        output1
@@ -104,6 +111,7 @@ compile flags input output
           output4   = Pass4.wrap_CGrammar        (Pass4.sem_CGrammar(Pass3.output_Syn_Grammar  output3)) Pass4.Inh_CGrammar {Pass4.options_Inh_CGrammar = flags'}
           output4a  = Pass4a.wrap_CGrammar       (Pass4a.sem_CGrammar(Pass3.output_Syn_Grammar output3)) Pass4a.Inh_CGrammar {Pass4a.options_Inh_CGrammar = flags'}
           output4b  = Pass4b.wrap_ExecutionPlan  (Pass4b.sem_ExecutionPlan grammar3a) Pass4b.Inh_ExecutionPlan {Pass4b.options_Inh_ExecutionPlan = flags', Pass4b.inhmap_Inh_ExecutionPlan = Pass3a.inhmap_Syn_Grammar output3a, Pass4b.synmap_Inh_ExecutionPlan = Pass3a.synmap_Syn_Grammar output3a, Pass4b.pragmaBlocks_Inh_ExecutionPlan = pragmaBlocksTxt, Pass4b.importBlocks_Inh_ExecutionPlan = importBlocksTxt, Pass4b.textBlocks_Inh_ExecutionPlan = textBlocksDoc, Pass4b.moduleHeader_Inh_ExecutionPlan = mkModuleHeader $ Pass1.moduleDecl_Syn_AG output1, Pass4b.mainName_Inh_ExecutionPlan = mkMainName mainName $ Pass1.moduleDecl_Syn_AG output1, Pass4b.mainFile_Inh_ExecutionPlan = mainFile, Pass4b.textBlockMap_Inh_ExecutionPlan = textBlockMap, Pass4b.mainBlocksDoc_Inh_ExecutionPlan = mainBlocksDoc,Pass4b.localAttrTypes_Inh_ExecutionPlan = Pass3a.localSigMap_Syn_Grammar output3a}
+          output4c  = Pass4c.wrap_ExecutionPlan  (Pass4c.sem_ExecutionPlan grammar3a) Pass4c.Inh_ExecutionPlan {Pass4c.options_Inh_ExecutionPlan = flags', Pass4c.inhmap_Inh_ExecutionPlan = Pass3a.inhmap_Syn_Grammar output3a, Pass4c.synmap_Inh_ExecutionPlan = Pass3a.synmap_Syn_Grammar output3a, Pass4c.mainName_Inh_ExecutionPlan = mkMainName mainName $ Pass1.moduleDecl_Syn_AG output1, Pass4c.mainFile_Inh_ExecutionPlan = mainFile, Pass4c.localAttrTypes_Inh_ExecutionPlan = Pass3a.localSigMap_Syn_Grammar output3a}
           output5   = Pass5.wrap_Program         (Pass5.sem_Program (Pass4.output_Syn_CGrammar output4)) Pass5.Inh_Program  {Pass5.options_Inh_Program  = flags', Pass5.pragmaBlocks_Inh_Program = pragmaBlocksTxt, Pass5.importBlocks_Inh_Program = importBlocksTxt, Pass5.textBlocks_Inh_Program = textBlocksDoc, Pass5.textBlockMap_Inh_Program = textBlockMap, Pass5.mainBlocksDoc_Inh_Program = mainBlocksDoc, Pass5.optionsLine_Inh_Program = optionsLine, Pass5.mainFile_Inh_Program = mainFile, Pass5.moduleHeader_Inh_Program = mkModuleHeader $ Pass1.moduleDecl_Syn_AG output1, Pass5.mainName_Inh_Program = mkMainName mainName $ Pass1.moduleDecl_Syn_AG output1}
           output5a  = Pass5a.wrap_Program        (Pass5a.sem_Program (Pass4.output_Syn_CGrammar output4)) Pass5a.Inh_Program { Pass5a.options_Inh_Program  = flags', Pass5a.textBlockMap_Inh_Program = textBlockMap }
           output6   = PrErr.wrap_Errors          (PrErr.sem_Errors                       errorsToReport) PrErr.Inh_Errors   {PrErr.options_Inh_Errors   = flags', PrErr.dups_Inh_Errors = [] }
@@ -124,7 +132,9 @@ compile flags input output
           furtherErrors    = if kennedyWarren flags'
                              then let errs3a = Pass3a.errors_Syn_Grammar output3a
                                   in if Seq.null errs3a
-                                     then toList ( Pass4b.errors_Syn_ExecutionPlan output4b )
+                                     then if ocaml flags' 
+                                          then toList ( Pass4c.errors_Syn_ExecutionPlan output4c )
+                                          else toList ( Pass4b.errors_Syn_ExecutionPlan output4b )
                                      else toList errs3a
                              else toList ( Pass3.errors_Syn_Grammar  output3
                                   Seq.>< Pass4.errors_Syn_CGrammar output4)
@@ -155,20 +165,22 @@ compile flags input output
           importBlocksTxt = vlist_sep "" . map addLocationPragma . concat . Map.elems $ importBlocks
           textBlocksDoc   = vlist_sep "" . map addLocationPragma . Map.findWithDefault [] (BlockOther, Nothing) $ textBlocks
           mainBlocksDoc   = vlist_sep "" . map addLocationPragma . Map.findWithDefault [] (BlockMain, Nothing) $ textBlocks
+          dataBlocksDoc   = vlist_sep "" . map addLocationPragma . Map.findWithDefault [] (BlockData, Nothing) $ textBlocks
+          recBlocksDoc    = vlist_sep "" . map addLocationPragma . Map.findWithDefault [] (BlockRec, Nothing) $ textBlocks
           pragmaBlocksTxt = unlines . concat . map fst  . concat . Map.elems $ pragmaBlocks
           textBlockMap    = Map.map (vlist_sep "" . map addLocationPragma) . Map.filterWithKey (\(_, at) _ -> at /= Nothing) $ textBlocks
 
-          outputfile = if null output then outputFile input else output
-          mainFile | null output = outputFile input
-                   | otherwise   = dropExtension output
-          mainName = dropExtension $ takeFileName input
+          outputfile = if null output then outputFile flags' inputfile else output
+          mainFile | null output = outputFile flags' inputfile
+                   | otherwise   = output
+          mainName = dropExtension $ takeFileName inputfile
 
           addLocationPragma :: ([String], Pos) -> PP_Doc
           addLocationPragma (strs, p)
-            | genLinePragmas flags'
-                = "{-# LINE" >#< pp (show (line p)) >#< show (file p) >#< "#-}" >-< vlist (map pp strs) >-< "{-# LINE" >#< ppWithLineNr (pp.show.(+1)) >#< show outputfile >#< "#-}"
-            | otherwise
-                = vlist (map pp strs)
+            | genLinePragmas flags' =
+                ppLinePragma flags' (line p) (file p) >-< vlist (map pp strs)
+                >-< ppWithLineNr (\l -> ppLinePragma flags' (l+1) outputfile)
+            | otherwise = vlist (map pp strs)
 
           optionsGHC = option (unbox flags') "-fglasgow-exts" ++ option (bangpats flags') "-XBangPatterns"
           option True s  = [s]
@@ -184,7 +196,6 @@ compile flags input output
           additionalWarnings = totalNrOfWarnings - nrOfWarningsToReport
           pluralS n = if n == 1 then "" else "s"
 
-
       (outAgi, ext) <-  --marcos
                      if genAspectAG flags'
                      then parseAGI flags (searchPath flags) (agiFile input)
@@ -200,20 +211,19 @@ compile flags input output
                                                                                                                , AspectAGDump.agi_Inh_Grammar      = agi
                                                                                                                , AspectAGDump.ext_Inh_Grammar      = ext' } --marcos
 
-
-      putStr . formatErrors $ PrErr.pp_Syn_Errors output6
+      printStr . formatErrors $ PrErr.pp_Syn_Errors output6
 
       if additionalErrors > 0
-       then putStr $ "\nPlus " ++ show additionalErrors ++ " more error" ++ pluralS additionalErrors ++
+       then printStr $ "\nPlus " ++ show additionalErrors ++ " more error" ++ pluralS additionalErrors ++
                      if additionalWarnings > 0
                      then " and " ++ show additionalWarnings ++ " more warning" ++ pluralS additionalWarnings ++ ".\n"
                      else ".\n"
        else if additionalWarnings > 0
-            then putStr $ "\nPlus " ++ show additionalWarnings ++ " more warning" ++ pluralS additionalWarnings ++ ".\n"
+            then printStr $ "\nPlus " ++ show additionalWarnings ++ " more warning" ++ pluralS additionalWarnings ++ ".\n"
             else return ()
 
       if not (null errorsToStopOn)  -- note: this may already run quite a part of the compilation...
-       then exitFailure
+       then failWith 1
        else
         do
            if genvisage flags'
@@ -227,9 +237,11 @@ compile flags input output
            if sepSemMods flags'
             then do -- alternative module gen
                     if kennedyWarren flags'
-                      then Pass4b.genIO_Syn_ExecutionPlan output4b
+                      then if ocaml flags' 
+                           then error "sepsemmods is not implemented for the ocaml output generation"
+                           else Pass4b.genIO_Syn_ExecutionPlan output4b
                       else Pass5.genIO_Syn_Program output5
-                    if not (null errorsToStopOn) then exitFailure else return ()
+                    if not (null errorsToStopOn) then failWith 1 else return ()
             else do -- conventional module gen
                     let doc
                          | visitorsOutput flags'
@@ -250,6 +262,8 @@ compile flags input output
                                     , AspectAGDump.imp_Syn_Grammar aspectAG
                                     , pp "\n\n{-- AspectAG Code --}\n\n"
                                     , AspectAGDump.pp_Syn_Grammar aspectAG
+                                    , dataBlocksDoc
+                                    , mainBlocksDoc
                                     , textBlocksDoc
                                     , if dumpgrammar flags'
                                       then vlist [ pp "{- Dump of AGI"
@@ -261,7 +275,43 @@ compile flags input output
                                                  ]
                                       else empty]
                          | kennedyWarren flags'
-                            = vlist [ Pass4b.warrenFlagsPP flags'
+                            = if ocaml flags'
+                              then vlist
+                                    [ text "(* generated by UUAG from" >#< mainFile >#< "*)"
+                                    , pp pragmaBlocksTxt
+                                    , text "(* module imports *)"
+                                    , pp importBlocksTxt
+                                    , Pass4c.modules_Syn_ExecutionPlan output4c
+                                    , text ""
+                                    , text "(* generated data types *)"
+                                    , text "module Data__ = struct"
+                                    , indent 2 $ vlist
+                                      [ text "type __generated_by_uuagc__ = Generated_by_uuagc__"
+                                      , Pass4c.datas_Syn_ExecutionPlan output4c
+                                      ]
+                                    , text "end"
+                                    , text "open Data__"
+                                    , text ""
+                                    , text "(* embedded data types *)"
+                                    , dataBlocksDoc
+                                    , text ""
+                                    , text "(* embedded utilty functions *)"
+                                    , textBlocksDoc
+                                    , text "(* generated evaluationcode *)"
+                                    , text "module Code__ = struct"
+                                    , indent 2 $ vlist
+                                      [ text "let rec __generated_by_uuagc__ = Generated_by_uuagc__"
+                                      , Pass4c.code_Syn_ExecutionPlan output4c
+                                      , recBlocksDoc
+                                      ]
+                                    , text "end"
+                                    , text "open Code__"
+                                    , text ""
+                                    , text "(* main code *)"
+                                    , mainBlocksDoc
+                                    ]
+                              else vlist
+                                    [ Pass4b.warrenFlagsPP flags'
                                     , pp pragmaBlocksTxt
                                     , pp $ if isNothing $ Pass1.moduleDecl_Syn_AG output1
                                            then moduleHeader flags' mainName Nothing
@@ -277,7 +327,10 @@ compile flags input output
                                                  , pp $ "import Control.Concurrent(newEmptyMVar,forkIO,putMVar,takeMVar)"]
                                       else vlist [ pp $ "import Control.Monad.Identity (Identity)"
                                                  , pp $ "import qualified Control.Monad.Identity" ]
+                                    , dataBlocksDoc
+                                    , mainBlocksDoc
                                     , textBlocksDoc
+                                    , recBlocksDoc
                                     --, pp $ "{-"
                                     --, Pass3a.depgraphs_Syn_Grammar output3a
                                     --, Pass3a.visitgraph_Syn_Grammar output3a
@@ -301,6 +354,8 @@ compile flags input output
                                               else []
                                             )
                                     , pp importBlocksTxt
+                                    , dataBlocksDoc
+                                    , mainBlocksDoc
                                     , textBlocksDoc
                                     , vlist $ if not (ocaml flags')
                                               then Pass5.output_Syn_Program  output5
@@ -327,16 +382,16 @@ compile flags input output
                     -- HACK: write statistics
                     let nAuto = Pass3.nAutoRules_Syn_Grammar output3
                         nExpl = Pass3.nExplicitRules_Syn_Grammar output3
-                        line  = input ++ "," ++ show nAuto ++ "," ++ show nExpl ++ "\r\n"
+                        line' = inputfile ++ "," ++ show nAuto ++ "," ++ show nExpl ++ "\r\n"
                     case statsFile flags' of
-                      Nothing   -> return ()
-                      Just file -> appendFile file line
-                    if not (null errorsToStopOn) then exitFailure else return ()
+                      Nothing -> return ()
+                      Just f  -> appendFile f line'
+                    if not (null errorsToStopOn) then failWith 1 else return ()
 
 
 
 formatErrors :: PP_Doc -> String
-formatErrors pp = disp pp 5000 ""
+formatErrors doc = disp doc 5000 ""
 
 
 message2error :: Message Token Pos -> Error
@@ -363,11 +418,6 @@ moduleHeader flags input export
          genExp Nothing _ = ""
          genExp (Just e) x = "(module " ++ x ++ ", module " ++ e ++ ")"
 
-inputFile :: String -> String
-inputFile name
-  | takeExtension name == ".ag" || takeExtension name == ".lag" = name
-  | otherwise                                                   = replaceExtension name "ag"
-
 --marcos
 agiFile :: String -> String
 agiFile name = replaceExtension name "agi"
@@ -375,8 +425,10 @@ agiFile name = replaceExtension name "agi"
 remAgi :: String -> String
 remAgi = dropExtension
 
-outputFile :: String -> String
-outputFile name = replaceExtension name "hs"
+outputFile :: Options -> String -> String
+outputFile opts name
+  | ocaml opts = replaceExtension name "ml"
+  | otherwise  = replaceExtension name "hs"
 
 defaultModuleName :: String -> String
 defaultModuleName = dropExtension
@@ -391,9 +443,9 @@ mkModuleHeader :: Maybe (String,String,String) -> String -> String -> String ->
 mkModuleHeader Nothing defaultName suffix _ _
   = "module " ++ defaultName ++ suffix ++ " where"
 mkModuleHeader (Just (name, exports, imports)) _ suffix addExports replaceExports
-  = "module " ++ name ++ suffix ++ exp ++ " where\n" ++ imports ++ "\n"
+  = "module " ++ name ++ suffix ++ ex ++ " where\n" ++ imports ++ "\n"
   where
-    exp = if null exports || (replaceExports && null addExports)
+    ex  = if null exports || (replaceExports && null addExports)
           then ""
           else if null addExports
                then "(" ++ exports ++ ")"
@@ -409,33 +461,34 @@ reportDeps flags files
 getDeps :: Options -> [String] -> IO [String]
 getDeps flags files
   = do results <- mapM (depsAG flags (searchPath flags)) files
-       let (fs, mesgs) = foldr combine ([],[]) results
+       let (fs, mesgs) = foldr comb ([],[]) results
        let errs = take (min 1 (wmaxerrs flags)) (map message2error mesgs)
        let ppErrs = PrErr.wrap_Errors (PrErr.sem_Errors errs) PrErr.Inh_Errors {PrErr.options_Inh_Errors = flags, PrErr.dups_Inh_Errors = []}
        if null errs
         then return fs
         else do putStr . formatErrors $ PrErr.pp_Syn_Errors ppErrs
-                exitFailure
+                failWithCode flags 1
+                return []
   where
-    combine :: ([a],[b]) -> ([a], [b]) -> ([a], [b])
-    combine (fs, mesgs) (fsr, mesgsr)
+    comb :: ([a],[b]) -> ([a], [b]) -> ([a], [b])
+    comb (fs, mesgs) (fsr, mesgsr)
       = (fs ++ fsr, mesgs ++ mesgsr)
 
 
 writeAttributeList :: String -> AttrMap -> IO ()
-writeAttributeList file mp
-  = writeFile file s
+writeAttributeList fileP mp
+  = writeFile fileP s
   where
     s = show $ map (\(x,y) -> (show x, y)) $ Map.toList $ Map.map (map (\(x,y) -> (show x, y)) . Map.toList . Map.map (map (\(x,y) -> (show x, show y)) . Set.toList)) $ mp
 
 readIrrefutableMap :: Options -> IO AttrMap
 readIrrefutableMap flags
   = case forceIrrefutables flags of
-      Just file -> do s <- readFile file
-                      seq (length s) (return ())
-                      let lists :: [(String,[(String,[(String, String)])])]
-                          lists = read s
-                      return $ Map.fromList [ (identifier n, Map.fromList [(identifier c, Set.fromList [ (identifier fld, identifier attr) | (fld,attr) <- ss ]) | (c,ss) <- cs ]) | (n,cs) <- lists ]
+      Just fileP -> do s <- readFile fileP
+                       seq (length s) (return ())
+                       let lists :: [(String,[(String,[(String, String)])])]
+                           lists = read s
+                       return $ Map.fromList [ (identifier n, Map.fromList [(identifier c, Set.fromList [ (identifier fld, identifier attr) | (fld,attr) <- ss ]) | (c,ss) <- cs ]) | (n,cs) <- lists ]
       Nothing   -> return Map.empty
 
 
