@@ -1,5 +1,4 @@
-{-# LANGUAGE MagicHash,
-             UnboxedTuples, FlexibleInstances #-}
+{-# LANGUAGE MagicHash, UnboxedTuples, FlexibleInstances #-}
 module Scanner where
 import GHC.Prim
 import TokenDef
@@ -39,8 +38,9 @@ input opts pos inp = Input pos
 type Lexer s = Pos -> String -> Maybe (s,Pos,String)
 
 scan :: Options -> Lexer Token
-scan opts
-  = scan
+scan opts p0
+  | column p0 == 1 = scanBeginOfLine p0
+  | otherwise      = scan p0
   where
     keywords' = if lcKeywords opts
                 then map (map toLower) keywords
@@ -98,6 +98,11 @@ scan opts
             scan' ('-'     :rs)      = (reserved "-" p, advc 1 p, rs)
             scan' ('*'     :rs)      = (reserved "*" p, advc 1 p, rs)
 
+            scan' ('\''    :rs) | ocaml opts =  -- note: ocaml type variables are encoded as 'TkTextnm' tokens
+              let (var,rest) = ident rs
+                  str = '\'' : var
+              in (valueToken TkTextnm str p, advc (length str) p, rest)
+
             scan' (x:rs) | isLower x = let (var,rest) = ident rs
                                            str        = (x:var)
                                            tok | str `elem` keywords' = reserved (mkKeyword str)
@@ -131,8 +136,9 @@ scan opts
 
 
 ident = span isValid
- where isValid x = isAlphaNum x || x =='_' || x == '\''
-lowercaseKeywords = ["loc","lhs", "inst", "optpragmas", "imports", "toplevel"]
+ where isValid x = isAlphaNum x || x == '_' || x == '\''
+
+lowercaseKeywords = ["loc","lhs", "inst", "optpragmas", "imports", "toplevel", "datablock", "recblock"]
 keywords = lowercaseKeywords ++
            [ "DATA", "EXT", "ATTR", "SEM","TYPE", "USE", "INCLUDE"
            , "EXTENDS" -- marcos
