@@ -1,6 +1,7 @@
-module UU.UUAGC (uuagc, uuagcMain) where
+module UU.UUAGC (uuagc, uuagcMain, compile, module Options) where
 
-import Ag (uuagcLib, uuagcExe)
+import Ag (uuagcLib, uuagcExe, compile)
+import Options
 
 import System.Exit (ExitCode(..))
 
@@ -8,4 +9,4 @@ uuagc :: [String] -> FilePath -> IO (ExitCode, [FilePath])
 uuagc = uuagcLib
 
 uuagcMain :: IO ()
-uuagcMain = uuagcExe
+uuagcMain = uuagcExe
\ No newline at end of file
